@@ -25,12 +25,14 @@ core file can be displayed with the following commands:
 """
 
 import abc
+import functools
 import typing
 
 import gdb
 
 from gdbmongo import stdlib_printers, stdlib_xmethods
-from gdbmongo.abseil_printers import AbslFlatHashMapPrinter, AbslNodeHashMapPrinter
+from gdbmongo.abseil_printers import (AbslFlatHashMapPrinter, AbslNodeHashMapPrinter,
+                                      AbslFlatHashSetPrinter)
 from gdbmongo.decorable_printer import DecorationContainerPrinter
 from gdbmongo.printer_protocol import PrettyPrinterProtocol, SupportsDisplayHint, SupportsToString
 from gdbmongo.string_data_printer import StdStringPrinter
@@ -66,6 +68,16 @@ class ServiceContextDecorationMixin(typing.Protocol):
         return cls.from_service_context(service_context)
 
 
+# pylint: disable-next=missing-class-docstring
+# pylint: disable-next=too-few-public-methods
+class ResourceCatalogGetter(typing.Protocol):
+
+    @abc.abstractmethod
+    def lookup_resource_name(self, res_id: gdb.Value, /) -> typing.Optional[str]:
+        """Return the database or collection namespace string of the resource."""
+        raise NotImplementedError
+
+
 # pylint: disable-next=missing-class-docstring
 # pylint: disable-next=too-few-public-methods
 class CollectionCatalogGetter(typing.Protocol):
@@ -81,7 +93,7 @@ class CollectionCatalogGetter(typing.Protocol):
 # We don't have to_string() or children() defined on _CollectionCatalogPrinter right now. Until we
 # have a sense of how else we might want to use the CollectionCatalog in GDB pretty printers, it is
 # probably best to mark the type as being internal to this module.
-class _CollectionCatalogPrinter(ServiceContextDecorationMixin):
+class _CollectionCatalogPrinter(ServiceContextDecorationMixin, ResourceCatalogGetter):
     """Pretty-printer for mongo::CollectionCatalog."""
 
     def __init__(self, val: gdb.Value, /) -> None:
@@ -156,6 +168,51 @@ class _CollectionCatalogPrinter(ServiceContextDecorationMixin):
         return cls(catalog)
 
 
+# We don't have to_string() or children() defined on _ResourceCatalogPrinter right now. Until we
+# have a sense of how else we might want to use the ResourceCatalog in GDB pretty printers, it is
+# probably best to mark the type as being internal to this module.
+class _ResourceCatalogPrinter(ServiceContextDecorationMixin, ResourceCatalogGetter):
+    """Pretty-printer for mongo::ResourceCatalog."""
+
+    def __init__(self, val: gdb.Value, /) -> None:
+        self.resources = val["_resources"]
+        self.val = val
+
+    def lookup_resource_name(self, res_id: gdb.Value, /) -> typing.Optional[str]:
+        """Return the database or collection namespace string of the resource."""
+        for (iter_res_id, iter_nss_set) in AbslNodeHashMapPrinter(self.resources).items():
+            if iter_res_id == res_id:
+                nss_set = iter_nss_set
+                break
+        else:
+            return None
+
+        namespaces = [nss for (_, nss) in AbslFlatHashSetPrinter(nss_set).children()]
+        return StdStringPrinter(namespaces[0]).string() if len(namespaces) == 1 else None
+
+    @classmethod
+    def from_service_context(cls, service_context: gdb.Value, /) -> "_ResourceCatalogPrinter":
+        """Return a _ResourceCatalogPrinter from its decoration on ServiceContext."""
+        try:
+            # The ResourceCatalog type was introduced and added as a decoration on the
+            # ServiceContext type as part of SERVER-67383 in MongoDB 6.2. Previously in MongoDB 6.0,
+            # the mapping of ResourceIds to collection and database names was managed through the
+            # CollectionCatalog.
+            resource_catalog_type = gdb.lookup_type("mongo::ResourceCatalog")
+        except gdb.error as err:
+            if not err.args[0].startswith("No type named "):
+                raise
+
+            raise ValueError(err.args[0]) from err
+
+        iterator = DecorationContainerPrinter(service_context["_decorations"]).children()
+        for (_, decoration) in iterator:
+            if decoration.type == resource_catalog_type:
+                return cls(decoration)
+
+        raise ValueError("Failed to locate ResourceCatalog decoration in ServiceContext")
+
+
 # We don't have to_string() or children() defined on _DatabaseShardingStateMapPrinter right now.
 # Until we have a sense of how else we might want to use the DatabaseShardingStateMap in GDB pretty
 # printers, it is probably best to mark the type as being internal to this module.
@@ -322,6 +379,12 @@ class ResourceIdPrinter(SupportsToString):
             ret += f", {label}"
 
             if "DatabaseShardingState" == label:
+                # The label for the DatabaseShardingState's ResourceMutex was changed as part of
+                # SERVER-70610 in MongoDB 6.2 where it now embeds the database name in the label.
+                # Previously in MongoDB 6.0, the labels for all DatabaseShardingStates'
+                # ResourceMutexes were the same generic "DatabaseShardingState" string and meant
+                # consulting the DatabaseShardingStateMap was the only way to know which
+                # ResourceMutex corresponded to which DatabaseShardingState.
                 try:
                     dss_map = _DatabaseShardingStateMapPrinter.from_global_service_context()
                 except ValueError:
@@ -332,40 +395,45 @@ class ResourceIdPrinter(SupportsToString):
 
         if self.resource_type in (gdb_lookup_value("mongo::RESOURCE_DATABASE"),
                                   gdb_lookup_value("mongo::RESOURCE_COLLECTION")):
-            collection_catalog = _CollectionCatalogPrinter.from_global_service_context()
-            if (nss := collection_catalog.lookup_resource_name(self.val)) is not None:
+            catalog: ResourceCatalogGetter
+
+            try:
+                catalog = _ResourceCatalogPrinter.from_global_service_context()
+            except ValueError:
+                catalog = _CollectionCatalogPrinter.from_global_service_context()
+
+            if (nss := catalog.lookup_resource_name(self.val)) is not None:
                 ret += f", {nss}"
 
+        if (self.resource_type == gdb_lookup_value("mongo::RESOURCE_GLOBAL")
+                and ResourceGlobalIdPrinter.is_type_defined()):
+            global_res_id = gdb.Value(self.hash_id).cast(gdb.lookup_type("mongo::ResourceGlobalId"))
+            ret += f", {global_res_id}"
+
         return ret
 
 
 # pylint: disable-next=too-few-public-methods
 class ResourceTypePrinter(SupportsToString):
     # pylint: disable=missing-function-docstring
-    """Pretty-printer for mongo::ResourceType"""
+    """Pretty-printer for mongo::ResourceType."""
 
-    # We duplicate the contents of mongo::ResourceTypeNames[] here for a couple reasons:
-    #
-    #   1. gdb.lookup_symbol("mongo::ResourceTypeNames") would OOM the GDB process when searching
-    #      for the symbol in a dynamically-linked mongod executable.
-    #
-    #   2. While gdb.lookup_static_symbol("mongo::ResourceTypeNames").value() would avoid the memory
-    #      pressure, that function is only available starting in the v4 toolchain. Moreover, the
-    #      `const char*` values are almost surely to have been optimized out anyway.
-    #
-    # If we do end up adding a new resource type to the middle of the hierarchy in a later version
-    # of the MongoDB Server (e.g. RSTL lock in MongoDB 4.2) then we'll need to come up with some
-    # scheme for detecting which set of names to use.
-    resource_type_names = (
-        "Invalid",
-        "ParallelBatchWriterMode",
-        "ReplicationStateTransition",
-        "Global",
-        "Database",
-        "Collection",
-        "Metadata",
-        "Mutex",
-    )
+    @functools.cached_property
+    def resource_type_names(self) -> typing.Tuple[str, ...]:
+        # We duplicate the contents of mongo::ResourceTypeNames[] here for a couple reasons:
+        #
+        #   1. gdb.lookup_symbol("mongo::ResourceTypeNames") would OOM the GDB process when
+        #      searching for the symbol in a dynamically-linked mongod executable.
+        #
+        #   2. While gdb.lookup_static_symbol("mongo::ResourceTypeNames").value() would avoid the
+        #      memory pressure, that function is only available starting in the v4 toolchain.
+        #      Moreover, the `const char*` values are almost surely to have been optimized out
+        #      anyway.
+        global_resource_names = ("Global", ) if ResourceGlobalIdPrinter.is_type_defined() else (
+            "ParallelBatchWriterMode", "ReplicationStateTransition", "Global")
+
+        return (("Invalid", ) + global_resource_names +
+                ("Database", "Collection", "Metadata", "Mutex"))
 
     def __init__(self, val: gdb.Value, /) -> None:
         self.val = val
@@ -374,10 +442,50 @@ class ResourceTypePrinter(SupportsToString):
         return self.resource_type_names[int(self.val)]
 
 
+class ResourceGlobalIdPrinter(SupportsToString):
+    # pylint: disable=missing-function-docstring
+    """Pretty-printer for mongo::ResourceGlobalId."""
+
+    # We duplicate the contents of mongo::ResourceGlobalIdNames[] for the same reasons described
+    # above in ResourceTypePrinter.
+    resource_global_id_names = (
+        "ParallelBatchWriterMode",
+        "FeatureCompatibilityVersion",
+        "ReplicationStateTransition",
+        "Global",
+    )
+
+    def __init__(self, val: gdb.Value, /) -> None:
+        self.val = val
+
+    def to_string(self) -> str:
+        return self.resource_global_id_names[int(self.val)]
+
+    @staticmethod
+    def is_type_defined() -> bool:
+        """Return True if the ResourceGlobalId type is defined, and return False otherwise."""
+        try:
+            # The ResourceGlobalId type was introduced as part of SERVER-65821 in MongoDB 6.0 and
+            # then subsequently backported to 4.4.15 and 5.0.10. resourceIdParallelBatchWriterMode
+            # and resourceIdReplicationStateTransitionLock, along with a new
+            # resourceIdFeatureCompatibilityVersion, all became distinct resources under the
+            # RESOURCE_GLOBAL ResourceType. The top-level RESOURCE_PBWM and RESOURCE_RSTL
+            # ResourceTypes were removed.
+            gdb.lookup_type("mongo::ResourceGlobalId")
+            return True
+        except gdb.error as err:
+            if not err.args[0].startswith("No type named "):
+                raise
+
+            return False
+
+
 def add_printers(pretty_printer: gdb.printing.RegexpCollectionPrettyPrinter, /) -> None:
     """Add the LockManager related printers to the pretty printer collection given."""
     pretty_printer.add_printer("mongo::LockManager", "^mongo::LockManager$", LockManagerPrinter)
     pretty_printer.add_printer("mongo::LockRequestList", "^mongo::LockRequestList$",
                                LockRequestListPrinter)
-    pretty_printer.add_printer("mongo::ResourceType", "^mongo::ResourceType$", ResourceTypePrinter)
     pretty_printer.add_printer("mongo::ResourceId", "^mongo::ResourceId$", ResourceIdPrinter)
+    pretty_printer.add_printer("mongo::ResourceType", "^mongo::ResourceType$", ResourceTypePrinter)
+    pretty_printer.add_printer("mongo::ResourceGlobalId", "^mongo::ResourceGlobalId$",
+                               ResourceGlobalIdPrinter)
